#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os;
import dircache;
import sys;
import urllib

class DimValue:
    def __init__(self, dim, value):
	self.dim = dim
	self.value = value
        self.dim.addValue(self)
    def __str__(self):
        return self.dim.val2str(self.value)
    def __repr__(self):
	return "DimValue(%s, %s)" % (repr(self.dim), repr(self.value))
    def htmlTableHeading(self):
	return self.dim.htmlTableHeading(self.value)

class DimValues(list):
    def replace(self, val):
        for i in xrange(len(self)):
            if self[i].dim == val.dim:
                self[i] = val
    def __add__(self, val):
        ret = DimValues(self)
        ret.append(val)
        return ret
    def __sub__(self, dim):
        result = DimValues(self)
        for v in self:
            if v.dim == dim:
                result.remove(v)
        return result
    def key(self):
        return tuple([v.value for v in self])

class Dimension(dict):
    def __init__(self, atype, name=None):
	self.type = atype
	if (name):
	    self.name = name
	else:
	    self.name = atype
        self.sortedKeys = []

    def __iter__(self):
	for i in xrange(len(self)):
	    yield self.getValue(i)
    def getValue(self, index):
        return self[self.sortedKeys[index]]

    def addValue(self, *values):
	for value in values:
	    if value not in self:
                if isinstance(value, DimValue):
                    self[value.value] = value
                else:
                    raise Exception("Unsupported usage of addValue")
                    #self[value] = DimValue(self, value)
	self.sortedKeys = self.keys()
	self.sortedKeys.sort()
    def val2str(self, v):
        return str(v)
    def htmlTableHeading(self, v):
	return self.val2str(v)
    def __str__(self):
        return self.name
    def __repr__(self):
        return "Dimension(%s)"%self.type

class DimensionKern(Dimension):
    def __init__(self):
	Dimension.__init__(self, 'gwkern', 'GW kernel')
    def htmlTableHeading(self, v):
	i=v.find(":")
	if i>0: kver=v[:i]
	else: kver=v
	return v+"<br><a href='config-%s'>config</a>"%(urllib.quote(kver))
    def versions(self):
	for v in self.values:
	    i=v.find(":")
	    if i>0: kver=v[:i]
	    else: kver=v
	    yield kver

class DimensionHostKern(Dimension):
    def __init__(self):
	Dimension.__init__(self, 'hostkern', 'Host kernel')
    def val2str(self, v):
        if v.find("host-") == 0:
            v = v[5:]
        return v
    def htmlTableHeading(self, v):
        v = self.val2str(v)
	i = v.find(":")
	if i>0: kver = v[:i]
        else: kver = v
	return v+"<br><a href='config-%s'>config</a>"%(urllib.quote(kver))
    def versions(self):
	for v in self.values:
	    i=v.find(":")
	    if i>0: kver=v[:i]
	    else: kver=v
	    yield kver

class DimensionTest(Dimension):
    def __init__(self):
	Dimension.__init__(self, 'test', 'Test')
    def htmlTableHeading(self, v):
	return v+"<br><a href='%s.sh.html'>source</a>"%(urllib.quote(v))

class DimensionTraffic(Dimension):
    def __init__(self):
	Dimension.__init__(self, 'traf', 'Traffic')
    def val2str(self, v):
        if v == "50":
            return "50%"
        elif v == "oneatatime":
            return "one message at a time"
        else:
            return v
    def htmlTableHeading(self, v):
        return self.val2str(v)
class Test:
    @classmethod
    def isOnPath(cls, path):
	f = os.path.join(path, '.results')
	return os.path.isfile(f)
    def __init__(self, path):
	self.path = path
        self.name = os.path.basename(path)
    def printThumbLink(self, file):
        thumb = self.path+'/thumb'
        try:
            imgs = [img for img in dircache.listdir(thumb)]
        except OSError:
            imgs = [ self.name + ".png" ]
        for img in imgs:
            print >>file, "<a href='%s/%s'><img src='%s/thumb/%s'></a>" % \
                  (urllib.quote(self.path), img, urllib.quote(self.path), img)

class Space(list):
    """List of Dimensions()s (order matters)"""
    def __init__(self, *dimensions):
        self.extend(list(dimensions))
    def iterValues(self):
        idx = [0 for i in xrange(len(self))]
        done=False
        while not done:
            values=DimValues()
            for i in xrange(len(self)):
                values.append(self[i].values()[idx[i]])
            yield values
            done=True
            for i in xrange(len(self)):
                idx[i] += 1
                if idx[i] < len(self[i]):
                    done=False
                    break
                idx[i] = 0
    def reorder(self, dimValues):
        reordered = DimValues()
        for d in self:
	    for v in dimValues:
		if v.dim == d:
		    reordered.append(v)
        return reordered
    def iterDimensionPairs(self):
	for i in xrange(len(self)):
	    for j in xrange(i+1, len(self)):
		yield (self[i], self[j])
		yield (self[j], self[i])
    def iterRemainingDimensions(self, dimensionPair):
	for d in self:
	    if d not in dimensionPair:
		yield d


class Tests(dict):
    """Represents all tests organized along several dimensions"""
    def __init__(self, rootpath, space):
	dict.__init__(self)
	self.space = space
	if (rootpath):
	    self.populate(rootpath)
    def getTest(self, key):
	if len(key) != len(self.space):
	    raise KeyError("The coordinates in key do not match the dimension of the space")
        realkey = self.space.reorder(key)
	return self[realkey.key()]

    def addTest(self, test, coordinates):
	if len(coordinates) != len(self.space):
	    raise KeyError("The number coordinates do not match the number of dimensions: " + str(coordinates))
	self[tuple(coordinates)] = test
	for i in xrange(len(coordinates)):
            DimValue(self.space[i], coordinates[i])

    def populate(self, rootpath):
	for root, dirs, files in os.walk(rootpath):
	    if (root.find(rootpath) == 0):
		coordinates = root[len(rootpath):]
	    else:
		coordinates = rootpath
	    if Test.isOnPath(root):
		self.addTest(Test(root), coordinates.split("/"))
    def generateHtml(self):
	for pair in self.space.iterDimensionPairs():
	    remDims = Space(*tuple([d for d in self.space.iterRemainingDimensions(pair)]))
	    for vals in remDims.iterValues():
		page = Page(pair, vals, self)
		page.generate()
	try:
	    os.remove("index.html")
	except OSError: pass
	os.symlink(page.getName(), "index.html")

	#os.system("source-highlight -d --output-dir=. ../*.sh")

class Page:
    def __init__(self, dimPair, valsOther, tests):
	self.dimy, self.dimx = dimPair
	self.dimOther = [v.dim for v in valsOther]
	self.valsOther = tests.space.reorder(valsOther)
	self.tests = tests
    def getName(self):
	return "%s-vs-%s-for-%s.html"%(self.dimy.type, self.dimx.type,
                                       "-".join([v.value for v in self.valsOther]))
    def generate(self):
	html = open(self.getName(), "w")
	title = "CAN gateway timing analysis" 
	print >> html, """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
<head>
<meta http-equiv="content-type" content="text/html; charset=utf-8" />
<title>%s</title>
<style>
img { border: 0; }
table { border-collapse: collapse; }
th, td { border: 1px solid lightgray; padding: 4px;}
</style>
</head>
<body>
<h1>%s</h1>"""  % (title, title)
        params = ["%s %s" % (v.dim, v) for v in self.valsOther]
        print >>html, "Results for:", ", ".join(params), "<hr />"
	for d in self.dimOther:
            print >>html, "%s: " % d
            print >>html, "<a href='%s'>X axis</a>, " % \
                Page((self.dimy, d), self.valsOther - d + self.dimx.getValue(0), self.tests).getName()
            print >>html, "<a href='%s'>Y axis</a>;&nbsp;&nbsp;" % \
                Page((d, self.dimx), self.valsOther - d + self.dimy.getValue(0), self.tests).getName()
            links = []
            for v in d.values():
                if v in self.valsOther:
                    links.append(str(v))
                else:
                    vv = DimValues(self.valsOther)
                    vv.replace(v)
                    links.append("<a href='%s'>%s</a>"%(urllib.quote(Page((self.dimy, self.dimx), vv, self.tests).getName()), str(v)))
            print >>html, " | ".join(links)
            print >>html, "<br>"
            try:
                print >>html, d.htmlPreamble()
            except Exception:
                pass

	print >>html, "<table><thead><tr><td>%s &rarr; <br />%s &darr;</td>" % (self.dimx.name, self.dimy.name)
	for x in self.dimx:
	    print >>html, "<th>%s</th>" % x.htmlTableHeading()
	print >>html, "</tr></thead>"
	for y in self.dimy:
	    print >>html, "<tr><th>%s</th>" % y.htmlTableHeading()

	    for x in self.dimx:
		print >>html, "<td>"
		idx = [x,y]
		idx.extend(self.valsOther)
                try:
                    test = tests.getTest(idx)
                    test.printThumbLink(html)
                except KeyError:
                    print >>html, "N/A"
                print >>html, "</td>"
	    print >>html, "</tr>"
	print >> html, """
</table>
<div style="font-size: small; color: gray; margin-top: 1em;">Authors: Michal Sojka, Pavel Píša, Copyright © 2010 Czech Technical University in Prague</div>
</body>
"""


if __name__ == "__main__":
    os.chdir(sys.argv[1])
    os.system("rm *.html")
    tests = Tests("./", Space(DimensionHostKern(), DimensionKern(), DimensionTraffic(), DimensionTest()))
    tests.generateHtml()
    sys.exit(0)
